// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/swag"
)

// DeleteDecisionResponse DeleteDecisionResponse
//
// swagger:model DeleteDecisionResponse
type DeleteDecisionResponse struct {

	// number of deleted decisions
	NbDeleted string `json:"nbDeleted,omitempty"`
}

// Validate validates this delete decision response
func (m *DeleteDecisionResponse) Validate(formats strfmt.Registry) error {
	return nil
}

// MarshalBinary interface implementation
func (m *DeleteDecisionResponse) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *DeleteDecisionResponse) UnmarshalBinary(b []byte) error {
	var res DeleteDecisionResponse
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
