// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"github.com/go-openapi/strfmt"
)

// AddAlertsResponse AddAlertsResponse
//
// swagger:model AddAlertsResponse
type AddAlertsResponse []string

// Validate validates this add alerts response
func (m AddAlertsResponse) Validate(formats strfmt.Registry) error {
	return nil
}
