// Code generated by entc, DO NOT EDIT.

package predicate

import (
	"github.com/facebook/ent/dialect/sql"
)

// Alert is the predicate function for alert builders.
type Alert func(*sql.Selector)

// Bouncer is the predicate function for bouncer builders.
type Bouncer func(*sql.Selector)

// Decision is the predicate function for decision builders.
type Decision func(*sql.Selector)

// Event is the predicate function for event builders.
type Event func(*sql.Selector)

// Machine is the predicate function for machine builders.
type Machine func(*sql.Selector)

// Meta is the predicate function for meta builders.
type Meta func(*sql.Selector)
