// Code generated by entc, DO NOT EDIT.

package runtime

// The schema-stitching logic is generated in github.com/crowdsecurity/crowdsec/pkg/database/ent/runtime.go

const (
	Version = "v0.5.4"                                          // Version of ent codegen.
	Sum     = "h1:kIf2BQUdRJ7XrlTXzCyJCg69ar1K1FjFR2UQWRo/M8M=" // Sum of ent codegen.
)
